<?php
# for debugging:
# header("content-type: text/xml");

include_once('funcs-upnp.php');
include_once('funcs-misc.php');
include_once('funcs-local.php'); # TODO: move include to switch
include('media-items.php');

# Parse the request from WDTVL
$requestRaw = file_get_contents('php://input');
if ($requestRaw != '') {
	$upnpRequest = _parseUPnPRequest($requestRaw);
	# DEBUG:	
	#_var_dump_to_file($requestRaw);
} else {
	echo 'No Request! wtf?';
	exit;
} # end if


# The objectId should be '0' for the root container or in URL format.
# eg 'umsp://plugins/apple-trailers?arg=val'
# or 'umsp://local/tmp/media/usb?random=true&sort_attr=name'
#
# The WDTVL usually requests a range of 64 items so the items of the last request are cached to speed up the scrolling.
# 
#
$items = array();
$cachedItems = _readCache($upnpRequest['objectid']);
if (isset($cachedItems)) {
	# cache hit
	$items = $cachedItems;
} else {
	# cache miss
	switch ($upnpRequest['objectid']) {
		case '0':
			# Root items:
			$items = $myMediaItems;
			$msgItems = _createMessageItems(array('UMSP v0.1.1 - Have fun!'));
			$items = array_merge($items, $msgItems);
			break;
			
		default:
			# The parse_url function returns an array in this format:
			# Array (
			# 	[scheme] => http
			# 	[host] => hostname
			# 	[user] => username
			# 	[pass] => password
			# 	[path] => /path
			# 	[query] => arg=value
			# 	[fragment] => anchor
			# )
			$reqObjectURL = parse_url($upnpRequest['objectid']);
			switch ($reqObjectURL['scheme']) {
				case 'umsp':
					switch ($reqObjectURL['host']) {
						case 'plugins':
							# handle plugins
							# call with parameters query and path
							$items = _callPlugin($reqObjectURL['path'], $reqObjectURL['query']);
							break;
						case 'local':
							# handle local content
							# call with parameters query and path
							$items = _localMain($reqObjectURL['path'], $reqObjectURL['query']);
							break;
					} # end switch
					break;
					# URL is not umsp:// try to load URL as DIDL-XML ??
			} # end switch
	} # end switch
} # end if

$totMatches = count($items);
if ($totMatches == 0) {
	$domDIDL = _createDIDL('');
	$numRet = 0;
} else {
	$dummy = _writeCache($upnpRequest['objectid'], $items);
	$slicedItems = array_slice($items, $upnpRequest['startingindex'], $upnpRequest['requestedcount']);
	$domDIDL = _createDIDL($slicedItems);
	$numRet = count($slicedItems);
}
# Build DIDL-XML from $myMediaItems array
$xmlDIDL = $domDIDL->saveXML();

# Build SOAP-XML reply from DIDL-XML and send it to WDTVL
$domSOAP = _createSOAPEnvelope($xmlDIDL, $numRet, $totMatches, '2');
echo $domSOAP->saveXML();


# --------------- FUNCTIONS -----------------------------------------------------------

function _callPlugin($prmPath, $prmQuery = null) {
	# TODO: make plugins config Ini based
	# prmPath is expected as '/pluginName'
	$tmpArr = explode('/', $prmPath, 3);
	if ($tmpArr[1] != '') {
		$pluginName = $tmpArr[1];
		#$pluginFile = __DIR__ . '/plugins/'. $pluginName . '.php';     TODO: change for PHP 5.3
		$pluginFile = './plugins/'. $pluginName . '.php';
		if (file_exists($pluginFile)) {
			# Load plugin file and call its _pluginMain function with query string as parameter
			# The plugin is expected to return an array of media items
			require $pluginFile;
			$pluginItems = _pluginMain($prmQuery);
		} else {
		} # end if
	} # end if
	return $pluginItems;
} # end function


function _createMessageItems($prmText) {
	# $prmText can be a single string or an array of multiple strings (lines)
	if (is_string($prmText)) { $prmText = array($prmText); }
	foreach ($prmText as $line) {
		if (trim($line) != '') {
		$retItems[] = array('id' 		=> 'msg_' . count($retItems),
							'dc:title'	=> $line,
						);
		} # end if
	} # end foreach
	return $retItems;
} # end function


function _readCache($prmId) {
	if ($prmId == '0') { return null; } # don't cache root
	$filename = '/tmp/umsp-cache';
	if (!is_readable($filename)) { return null; }
	$handle = fopen($filename, 'r');
	$rawContent = fread($handle, filesize ($filename));
	fclose($handle);
	$cacheData = unserialize($rawContent);
	if ((is_array($cacheData['data'])) && ($cacheData['id'] == $prmId)) {
		return $cacheData['data'];
	} else {
		return null;
	} # end if
} # end function


function _writeCache($prmId, $prmDataArray) {
	if (!is_array($prmDataArray)) { return false; }
	if ($prmId == '0') { return false; } # don't cache root
	$filename = '/tmp/umsp-cache';
	$handle = fopen($filename, 'w');
	$rawContent = serialize(array('id' => $prmId, 'data' => $prmDataArray));
	$success = fwrite($handle, $rawContent);
	fclose ($handle);
	return $success;
} # end function

?>